/**
 * 
 *      Unfortunately, when the PresetManagers were created
 *      to replace Looks with Appearances etc, they included no events/handlers
 *      or callbacks, leaving us generally unable to listen for save/load events.
 *      
 *      however, while it may not call any actual event before saving, 
 *      the preset manager does do a sweep of any unusual or "dynamic" storables
 *      that are nested in the appearance transforms, and it does this before saving
 *      the 'usual' data.
 *      
 *      the listener we make here becomes a child of one of the objects that are sweeped,
 *      resulting in a GetJSON() call when the save begins. We return an empty string,
 *      but use that as a signal that the appearance is beginning to be saved,
 *      whereupon we can restore the morph visibility and resync the ui,
 *      so that the morphs are saved properly.
 *      
 *      QUITE the hack. open to suggestions of alternatives. 
 * 
 */



namespace GossamerVR.MorphFilterPlus
{

    using System;
    using UnityEngine;
    using SimpleJSON;


    /// <summary>
    /// Created by the morph filter main script
    /// to be a child of a person, in such a way as to detect
    /// when an appearance save begins. This allows it to
    /// tell the morph filter to revert the morph visibility
    /// so that they are saved properly in the file.
    /// </summary>
    public class AppearanceListener: JSONStorable
    {
        public event EventHandler GetJSONEvent;

        /// <summary>
        /// The person this listener is a descendent of in hierarchy.
        /// used to access the upgrade table in the main script,
        /// or to find other related transforms.
        /// </summary>
        public Atom Owner;

        /// <summary>
        /// called when the appearance is to be stored on the character we are a child of.
        /// this alerts the main script of the event (to restore morphs) and then
        /// reports back the geometry's JSON.
        /// </summary>
        public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
        {
            //SuperController.LogMessage("Before: "+ this.GetComponentInParent<DAZCharacterSelector>().GetJSON(includePhysical, includeAppearance, forceStore).ToString());

            JSONClass json = new JSONClass();
            json["GVR_IGNORE"] = "empty";

            // the main script is subscribed to us. we don't need to know who or what it is.
            // but we trust that after this event, our morphs will be ready to save.
            this.GetJSONEvent?.Invoke(this, EventArgs.Empty);

            // due to the order of saving, we know geometry + morphs is already committed.
            // but due to the dictionary nature of JSON, we can overwrite it by sending it again.
            // (now with the morphs that were hidden)
            //SuperController.LogMessage("After: " + this.GetComponentInParent<DAZCharacterSelector>().GetJSON(includePhysical, includeAppearance, forceStore).ToString());
            return this.GetComponentInParent<DAZCharacterSelector>().GetJSON(includePhysical, includeAppearance, forceStore);

            

        }

        // this script looks small but JSONStorable is actually pretty big.
        // it's an unfortunate overhead that I hope to find a better solution for.

    }




}